/**************************************************************\
* ReadMe.h for xlatHinc Version 0.9.8.0                        *
* Copyright  2003-2008 Wayne J. Radburn                       *
*                                                              *
* This file explains what xlatHinc does and provides an example*
* of what it will and will not translate in typical H files.   *
* Anything else encountered either gets ignored, or you may    *
* need to edit or delete the attempted translation.            *
*                                                              *
* Lines with a shaded background will not get saved to the INC *
* file. xlatHinc uses lighter shading for those lines that     *
* generally can be ignored, while a darker shading indicates   *
* lines that were ignored and possibly need further attention  *
* or were ignored with the #if processing described below.     *
*                                                              *
* The File menu contains two Open commands, one for H files    *
* and one for INC files, since you will most likely find these *
* files in seperate folders. The paths to these folders get    *
* saved in the Registry along with other display options (such *
* as size, position, font). You can open files of either type  *
* from either folder and this will not change the paths to     *
* these folders. However, a file placed on the command line    *
* or dropped into xlatHinc will change the path to these       *
* folders based on the H or INC file type.                     *
*                                                              *
* Note that when xlatHinc is already running, it will accept   *
* drag-and-drop of multiple files. After a prompt to confirm   *
* the INC save directory (set from a previous Open or Save),   *
* xlatHinc then processes each of these files using the        *
* #if "Continue" method for conditional compilation (see below)*
* and then automatically saves the resulting INC file.         *
*                                                              *
* The Edit menu contains a Copy command which will copy the    *
* current line or symbol definition (multiple lines for        *
* structures) to the clipboard. The Copy Function command will *
* take an assumed STDCALL (see DDQ option below) function like *
*                                                              *
* ;FunctionName(Param_1,Param_2,Param_3,...,Param_N)           *
*                                                              *
* and copy this to the clipboard as commented tabbed text like *
*                                                              *
*    push    0       ;Param_N                                  *
*    ;       ;       ;                                         *
*    push    0       ;Param_3                                  *
*    push    0       ;Param_2                                  *
*    push    0       ;Param_1                                  *
*    call    [FunctionName]  ;INC_FileName                     *
*                                                              *
* or with the last two lines modified as follows for Interface *
* function calls                                               *
*                                                              *
*    push    ebx     ;This                                     *
*    mov     eax,[ebx]                                         *
*    call    [eax+InterfaceName.FunctionName]  ;INC_FileName   *
*                                                              *
* which assumes that you have already initialized ebx to the   *
* This pointer for the Interface.                              *
*                                                              *
* The View menu contains commands for showing an alphabetical  *
* listing of the equates, structures, and/or function symbol   *
* names defined in the opened H or INC file. Selecting a       *
* symbol will bring that symbol definition into view in the    *
* INC file.                                                    *
*                                                              *
* The Options menu has DSS and ##AW options for a translation  *
* to an ANSI/Unicode switchable INC file instead of a Unicode  *
* translation. This requires the following code at the start   *
* of your source file or main include file:                    *
*                                                              *
* #ifdef UNICODE                                               *
* DSS = DUS                                                    *
* AW = W                                                       *
* S = 2                                                        *
* #else                                                        *
* DSS = DB                                                     *
* AW = A                                                       *
* S = 1                                                        *
* #endif                                                       *
*                                                              *
* The Options menu also has a DDQ option for translation of    *
* pointers and handles to a 32/64-bit switchable INC file      *
* instead of a 32-bit translation. This requires the following *
* at the start of your source file or main include file:       *
*                                                              *
* #if x64                                                      *
* DDQ = DQ                                                     *
* #else                                                        *
* DDQ = DD                                                     *
* #endif                                                       *
*                                                              *
* The DDQ option also modifies the Copy Function command to    *
* use ARG INVOKE and 64-bit registers.                         *
*                                                              *
* When viewing an INC file, #if... #else #endif and #include   *
* will appear in red, equates in gray, structures in the       *
* system window text colour, and functions in blue.            *
*                                                              *
* Although you can not see an example of this here, if you     *
* place leading comments in your INC file, they will appear    *
* green in two possible formats:                               *
*                                                              *
*  ;  single semi-colon for dark green text                    *
*  ;; double for dark green text on light green background     *
*                                                              *
* Leading blank comments will appear in two possible formats:  *
*                                                              *
*  ;  single semi-colon for a black horizontal line            *
*  ;; double for dark green background                         *
*                                                              *
\**************************************************************/


//--------------------------------------------------------------
// TRANSLATION FOR CONDITIONAL COMPILATION AND PREPROCESSING
//
// A dialog box will appear with options 0 to 3 to select how to
// process an #if... statement if that processing information
// has not already been added to the list in xlatHinc.ini. The
// wildcard character * can be used as the last character on a
// line to simplify repetitive similar entries on this list.
//
// The H file will usually scroll so that you may see the
// encountered #if... statement on the top line.
//
// Option 0 will ignore the complete #if... #else #endif block.
// Option 1 will translate just the  if  statements and
//          ignore the #if... #else  #endif lines.
// Option 2 will translate just the else statements and
//          ignore the #if... #else  #endif lines.
// Option 3 will translate the entire #if... #else #endif block
//          keeping the #if... #else  #endif lines.
//
// "Process" uses the option selected and advances to the next
//      #if statement.
// "Add to List" adds the option selected with the if statement
//      to the list.
// "Continue" processes the rest of the file using the current
//      list, with Option 3 selected for those #if... statements
//      not found on the list. A drag-and-drop operation with
//      multiple files will automatically use this method.
//
//--------------------------------------------------------------

#include	<File.h>
# include	"File.h"
#include	<pshpackN.h> // These and a few other files get ignored
#include	"poppack.h"

#error	IGNORE_ALL_OTHER_PREPROCESSING
#pragma
#undef
#line
#

//--------------------------------------------------------------
// TRANSLATION FOR EQUATES
//
// More complex expressions may required editing since
// brackets and spaces generally get removed.
//
// Occasionally an equate has an ANSI and Unicode value.
// xlatHinc keeps both and a default translation of the
// conditional code will keep the Unicode VALUE = VALUEW.
//
// Using the ##AW Option will translate this instead to the
// ANSI/Unicode switchable VALUE = VALUE##AW.
//
//--------------------------------------------------------------

#define	VALUEA (0x1)
#define	VALUEW (0x2)
# ifdef	UNICODE
#    define	VALUE VALUEW	// = VALUEW or optional VALUE##AW
#  else
#    define	VALUE VALUEA
# endif

#define TEXT_A "ANSI"
#define TEXT_W L"UNICODE"
#define TEXT_T TEXT("TEXT")

#define	VALUE_9 (VALUE + 1234U )
#define	VALUE_8 (VALUE- 0xABCDL)
#define	VALUE_7 (VALUE *1234L)
#define	VALUE_6 (VALUE)
#define	VALUE_5 (0UL)
#define	VALUE_4 (~0LL)
#define	VALUE_3 (0X0ui64)
#define	VALUE_2 (5.43e+21f)
#define	VALUE_1 (VALUE1 |      \
	        VALUE2 &		\
	        ~VALUE3)

#define	BITSHIFT_1	1 << 0				// Handles most bit shifting
#define	BITSHIFT_0	1 >> BITSHIFT_1
#define	BITSHIFT_2	(BITSHIFT_1 << BITSHIFT_1)
#define	BITSHIFT_4	(0x01 << 0x2)
#define	BITSHIFT_8	(BITSHIFT_2 << \
					(BITSHIFT_4 >> BITSHIFT_1))
#define	BITSHIFT_A	('A' << BITSHIFT_8)
#define	BITSHIFT_X	(VALUE << BITSHIFT)
#define	BITSHIFT_64	(1 << 34) | (12345678h << 16)

#define	SIZE_OF1 sizeof(TYPE)
#define	SIZE_OF3 sizeof( TYPEA) - sizeof(TYPEB ) + sizeof( TYPEC )
#define	SIZE_OF2 sizeof(TYPEA)/sizeof(TYPEB)

#define	SYMBOL_C (IGNORE)  1234			// Ignored text near brackets
#define	SYMBOL_D ((IGNORE) 1234)		// like this in most cases will
#define	SYMBOL_F (void IGNORE) -1234	// result in a proper translation
#define	SYMBOL_E (unsigned IGNORE) 1234
#define	SYMBOL_M	IGNORE (1234)		// This case often works, possible macro exceptions...

#define	MACRO(Arg)	IGNORE (Arg)				// Macro definitions are ignored
#define	SOMETHING	DEFINE_GUIDNAMED(SOMETHING)	// Some macros are ignored
DEFINE_SOMEMACRO(0x1,0x2,0x3);					// DEFINE_... ignored, some exceptions with GUIDs below
DECLARE_SOMEMACRO(0x1,0x2,0x3);					// DECLARE_... ignored, except DECLARE_INTERFACE_...
C_ASSERT(IGNORE);
#define	MACRO_00	IGNORED_WITH(More,Than,One,Arg)
#define	MACRO_01	MAKE_HRESULT(1,fac,code)			// These macros and many of their variations are translated
#define	MACRO_07	\
	MAKE_SCODE(SEVERITY_ERROR,fac,code)
#define	MACRO_08	CTL_CODE(DeviceType,Function,Method,Access)
#define MACRO_09	CLUSCTL_RESOURCE_CODE(Function)
#define	MACRO_10	_WSAIOR(x,y)
#define	MACRO_11	_IOR(x,y,ULONG)
#define	MACRO_12	SCARD_ATTR_VALUE(Class,Tag)			// A few other macros are also translated
#define	MACRO_13	CLUSPROP_SYNTAX_VALUE(type,format)
#define	MACRO_14	CLCTL_EXTERNAL_CODE(Function,Access,Modify)
#define	MACRO_15	CLCTL_INTERNAL_CODE(Function,Access,Modify)
#define	MACRO_16	PROP_TAG(Type,ID)
#define	MACRO_17	MQCLASS_CODE(1,1,code)
#define	MACRO_18	DXVAColorMask(bits,base)
#define	MACRO_19	NWF_DEFINE_OID(Seq,o,m)
#define	MACRO_20	BIT(a)|BIT(b)
#define	MACRO_21	MAKELCID(lgid,srtid)
#define	MACRO_22	MAKELANGID(p,s)
#define	MACRO_21_2	MAKELCID(MAKELANGID(p,s),srtid)
#define	MACRO_04	MAKEFOURCC('F','O','U','R')
#define	MACRO_03	(mmioFOURCC ('F', 'O' ,'U' , 'R'))
#define	MACRO_02	aviTWOCC('T', 'O')
#define	MACRO_05	FCC('FOUR')
#define	MACRO_06	MAKE_IDENTIFIER('F','O','U','R')

#define	SYMBOL_IGNORE				// value required
#define	SYMBOL_IGNORE	__something	// leading underscores
#define	SYMBOL_IGNORE	WINAPI
#define	INTERFACE		Iignore
#define	tagSOMETHING	tagIGNORE	// tag...
#define	_SOMETHING		IGNORE		// leading underscore
#define	LPSOMETHING		LPIGNORE	// LP...
#define	LPD_SOMETHING	0xABCD		// LP... exceptions
#define	LPM_SOMETHING	0xABCD
#define	LPV_SOMETHING	0xABCD
#define	LPSTR_TEXTCALLBACK	-1

#define	ISOMETHING {0x12345678,0x1234,0x1234,0x11,0x22,0x33,0x44,0x55,0x66,0x77,0x88}
#define	ISOMETHINGELSE \
		{0x12345678,0x1234,0x1234,\
		{0x11,0x22,0x33,0x44,0x55,0x66,0x77,0x88}}


//--------------------------------------------------------------
// TRANSLATION FOR ENUMERATIONS
//--------------------------------------------------------------

enum _ENUM_N
{	ENUM_2n = -2,
	ENUM_1n,
	ENUM_0n	};

typedef enum tagENUM_D
{	ENUM_0d
	,ENUM_1d
	,ENUM_2d};

typedef enum ENUM_H {   // Comment
	ENUM_9h = 0x9,
	ENUM_Ah,
	ENUM_Bh
};

typedef
enum _ENUM_V { ENUM_0v = VALUE,

	ENUM_1v, ENUM_2v,	// Comment, Comment
	ENUM_M	=
			MAKE_HRESULT(1,fac,code),
	ENUM_OR		// Comment
			= (ENUM_0v |
				ENUM_1v |	// Comment
				ENUM_2v)
} ENUM_V;

typedef enum
tagENUM_SIMPLIFY
{	ENUM_Fs = 0,
	ENUM_0s = ENUM_Fs,
	ENUM_1s =	ENUM_0s + 1,
	ENUM_2s		= ENUM_1s + 1,
	ENUM_3s		= ( ENUM_2s + 1 ),
	ENUM_Ls		= ENUM_3s
	MAKE_ENUM(Method, Interface),
	MAKE_ROUTE_PROTOCOL(suffix, value)
	} ENUM_S;


//--------------------------------------------------------------
// TRANSLATION FOR FUNCTIONS
//
// xlatHinc favors a Unicode translation and ignores an ANSI
// function definition. The default translation of the
// conditional code will keep Function = FunctionW.
//
// Using the ##AW Option, this will instead translate to the
// ANSI/Unicode switchable Function = Function##AW.
//
//--------------------------------------------------------------

FunctionA(void);			// ANSI - Ignored due to trailing A
FunctionW(void);			// Unicode - Removes trailing W
#ifdef UNICODE
#define	Function FunctionW	// = FunctionW or optional Function##AW
#else
#define	Function FunctionA
#endif

FunctionB(void);
SOMETHING(SOMETHINGELSE) FunctionE(Param1, *Param2, Param3 *, *) throw();
SOMETHING(SOMETHINGELSE FunctionD) (SOMEMORE Param1, *Param2, Param3 *, *);
SOMETHING FunctionF(
	IN	Param1,
	OUT	Param2,
	IN OUT	Param3 * OPTIONAL,
	*
);
__bcount(Param1) /* Comment */ SOMETHING FunctionC
	(__in	DWORD	Param1,		// Comment
	__in_xcount  (strlen(Param2) + 1) /* Comment */__out_data_source(FILE)
	DWORD  Param2,
	/* Comment */ __inout	DWORD	Param3 * OPTIONAL,
	*);

inline Ignored(
		Param1)
{
	return 0;
}


//--------------------------------------------------------------
// TRANSLATION FOR STRUCTURES
//
// xlatHinc favors a Unicode translation and ignores an ANSI
// structure definition. In most cases, there is no difference
// between the two versions with string pointers defined as DD.
//
// In the case of strings defined as a structure element, WCHAR
// and TCHAR will translate to DW.
//
// Using the DSS Option, xlatHinc will instead translate these
// to the ANSI/Unicode switchable DSS.
//
//--------------------------------------------------------------

struct {				// Removes trailing W
	WCHAR	String;		// DW or optional DSS
} UNICODE_1W;
struct {				// Removes trailing _W
	TCHAR	String;		// DW or optional DSS
} UNICODE_2_W;
struct {
} EXCEPTION_DRAW;
struct {
} EXCEPTION_VIEW;
struct {
} EXCEPTION_DOW;
struct {
} EXCEPTION_ROW;

struct {				// ANSI - Ignored
	CHAR	String;
}
ANSI_IGNORE_A;

struct{
} EXCEPTION_DATA;
struct {
} EXCEPTION_AREA;
struct {
} EXCEPTION_PARA;
struct {
} EXCEPTION_CIDA;
struct {
} EXCEPTION_ALPHA;
struct {
} EXCEPTION_ANTENNA;

struct _NO_UNDERSCORE{		// Remove leading underscore
};
struct tagNO_TAG_1 {		// Remove tag variations
};
struct _tagNO_TAG_2 {
};
struct tag_NO_TAG_3 {
};
struct _tag_NO_TAG_4 {
};
struct TAG(NO_TAG_5) {
};

typedef /* Comment */ struct tagPOINTERS
{
	PVOID	HandledP;		/* Most P... types get handled as
							   DD or optional DDQ */
	ANYTYPE*    AlwaysP;	// *
	ANYTYPE	*   AlwaysP;	// *
	ANYTYPE FAR* AlwaysP;	// *
	ANYTYPE		*AlwaysP;	// *
	LPTYPE	AlwaysP;		// LP...
	ANYTYPE	lpAlwaysP;		// lp...
	PFNTYPE	AlwaysP;		// PFN...
	ANYTYPE	pfnAlwaysP;		// pfn...
	ANYTYPE	pAlwaysP;		// p... with Uppercase for next character
	DWORD_PTR	AlwaysP;
	SIZE_T		AlwaysP;
	POINTER_32	AlwaysDD;
	POINTER_64	AlwaysDQ;
} POINTERS;

typedef struct tagHANDLES
{
	HANDLE					// Most H... types get handled as
			HandledH;		// DD or optional DDQ
	HFILE	AlwaysDD;
	HRESULT	AlwaysDD;
} HANDLES;

typedef struct common_misc
{
	CHAR	HandledB;		// Most other common
	SHORT	HandledW;		// types get handled too
	ULONG	HandledD;
	DOUBLE	HandledQ;

	LRESULT	Handled;

	ENUM_N	Handled;		// Handled if enum defined
	ENUM_D	Handled;		// within the same H file
	ENUM_H	Handled;
	ENUM_V	Handled;
	ENUM_S	Handled;

	UNICODE_1W	RemoveW ;
	UNICODE_2_W	Remove_W ;
	UNICODE_EXCEPTION_DRAW	KeepDraw;
	UNICODE_EXCEPTION_VIEW	KeepView;
	UNICODE_EXCEPTION_DOW	KeepDow;
	UNICODE_EXCEPTION_ROW	KeepRow;

	/* Comment */ DWORD IgnoreComment;		/* Comment *
											 * Comment */
	IN OUT CONST OPTIONAL	DWORD IgnoreInOutConstOpt OPTIONAL;
	__in __out __inout DWORD IgnoreAnnotations;

	OUT DWORD
	(WINAPI *Functioning)(
		IN	Param1,
		OUT	Param2
		);
	DWORD	Multi1,		// Comment
			Multi2;
	DWORD	Multi3, Multi4, Multi5;
	BYTE	Multi6; WORD Multi7;

	TYPE_NOMEMBERNAME;} COMMON_MISC;

struct DUPS
{										// Comment
	BYTE	KeepDUPdec[ 1234 ];
	BYTE	KeepDUPhex[0xABCD];
	BYTE	KeepDUPval[ VALUE];
	DWORD	MultiDimV[VALUE1][VALUE2];
	DWORD	MultiDimN[16][0x10];
	UCHAR	Buffer[ (256 * 2) + 4 ];	// Comment
	DWORD	IgnoreDUP0[ 0 ];
	WORD	IgnoreDUP1[1];
	BYTE	IgnoreDUP[];
	DWORD   (*IgnoreDUP)[];
	DWORD	IgnoreDUP[ANYSIZE_ARRAY];
	DWORD	IgnoreDUP[ANY_SIZE];
	};

struct {
	BYTE	Partial1;
	BYTE	fpB1:1;
	BYTE	fpB2:2;
	WORD	Partial2;
	WORD	fpW1:10;
	WORD	fpW2:4 ;
	DWORD	Full3;
	DWORD	fD1:20;
	DWORD	fD2:10 ;
	DWORD	fD3: 2 ;
	ULONGLONG	fQ1: 40;
	ULONGLONG	fQ2: 20;
	ULONGLONG	fQ3:  4;
	WORD	fM1a:4,
			fM1b:4,
			fM1c:8;
	BYTE	fM2a:2, fM2b:2, fM2c:4;
} BITFIELDS, *BITFIELDS, *LPBITFIELDS;

typedef struct tagNESTED
{
	DWORD	a;
	struct
	{
		DWORD	b;
		DWORD	c;
		struct
		{
			DWORD	d;
			DWORD	e;
		};
	} NamedStruct;
	DWORD	f;




	struct	{BYTE g,h; WORD i;};
 	struct SNAME	j;
	struct    _SNAME	k;	// Removes underscore
	struct  tagSNAME	l;	// Removes tag variations
	struct ANYTYPE* m;
} NESTED;

typedef union tagUNIONS
{
	union	{
		DWORD	a;
		DWORD	b;
		DWORD	c;
		struct  {
			DWORD	d;
			DWORD	e;
		};
	} DUMMYUNIONNAME;	// Nameless

	union	{
		struct	{
			DWORD	f;
			DWORD	g;
			union	{
				DWORD	h;
				DWORD	i;
				DWORD	j;
			} NamedUnion;
		};
		DWORD	k;
		DWORD	l;
	} u;				// Nameless


	struct	{
		DWORD	m;
		DWORD	n;
		union	{
			DWORD	o;
			DWORD	p;
		} u1;			// Nameless
	};




	union	{BYTE q,r; WORD s;};
	union UNAME t;
	union _UNAME u;		// Removes underscore
	union _tag_UNAME v;	// Removes tag variations
	union ANYTYPE* w;
} UNIONS;

typedef IGNORE IGNORED;
typedef struct IGNORE IGNORED;


//--------------------------------------------------------------
// TRANSLATION FOR INTERFACES
//
// Combines Structure and Function processing
//--------------------------------------------------------------

typedef struct ISeeVtbl		// Removes Vtbl
{
	BEGIN_INTERFACE
	HRESULT ( STDMETHODCALLTYPE *QueryInterface )(
		ISee * This,
		REFIID riid,
		void * ppvObj);
	ULONG ( STDMETHODCALLTYPE *AddRef)(
		ISee * This);
	ULONG ( STDMETHODCALLTYPE *Release)(
		ISee * This);

	HRESULT ( STDMETHODCALLTYPE *Way )(
		HRESULT ( STDMETHODCALLTYPE *Too__Much )(
			UINT Param1,
			UINT Param2);
	END_INTERFACE
} ISeeVtbl;

DECLARE_INTERFACE_(IDo, IUnknown)
{
    // *** IUnknown methods ***
    STDMETHOD(QueryInterface) (THIS_ REFIID riid, LPVOID * ppvObj) PURE;
    STDMETHOD_(ULONG,AddRef) (THIS)  PURE;
    STDMETHOD_(ULONG,Release) (THIS) PURE;

    // *** IWill methods ***
    STDMETHOD(This) (	                	// Comment
			THIS_
			ANYTHING /* Comment */ pSomething
			) PURE;
    STDMETHOD(That) (THIS_ ANYTHINGELSE fSomethingElse,
							PSTUFF *pStuff,
							BOOL fDefault /* in */
							) PURE;
    STDMETHOD(TheOtherThing) (THIS_ PMORESTUFF pMoreStuff,
							DWORD dwFlags,	// Comment
			/* Comment */	LPSTR *pszWhy,
			IN OUT			LPSTR *pszWhyNot OPTIONAL,
			__inout			LPSTR *pszWhen ) PURE;
};


//--------------------------------------------------------------
// TRANSLATION OF A GUIDs and UUIDs
//--------------------------------------------------------------

typedef /* COMMENT */ DECLSPEC_UUID("12345678-ABCD-1234-1122-334455667788")
enum
{	ENUM_0,
	ENUM_1
}	ENUMNAME;

typedef /* COMMENT */ DECLSPEC_UUID("12345678-ABCD-1234-1122-334455667788") struct
{
	DWORD	Member_0;
	DWORD	Member_1;
}	STRUCTNAME;

DEFINE_GUID(ISomeThing  ,0x12345678,  0xABCD,0x1234,
	0x11,0x22,0x33,0x44,0x55,0x66,0x77,0x88)
DEFINE_GUID(  /*  12345678-ABCD-1234-1122-334455667788 */
	ISameThing,
	0x12345678,
	0xABCD,
	0x1234,
	0x11,0x22,0x33,0x44,0x55,0x66,0x77,0x88)
DEFINE_OLEGUID (IThing, // Comment
	0x12345678,0xABCD,0x1234)
DEFINE_MEDIATYPE_GUID( ISimilarThing, FCC('FOUR')  );
DEFINE_DAOGUID (CLSID_Thing, 0x12345678)
DEFINE_AVIGUID(IID_Thing, 0x12345678, 0xABCD, 0x1234);
DEFINE_CODECAPI_GUID( GUID_Thing, "12345678-ABCD-1234-1122-334455667788", 0x12345678, 0xABCD,0x1234, 0x11,0x22,0x33,0x44,0x55,0x66,0x77,0x88 )
DEFINE_KNOWN_FOLDER(FOLDERID_Thing, 0x12345678, 0xABCD, 0x1234, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88);


interface DECLSPEC_UUID("12345678-ABCD-1234-1122-334455667788")
	   INoThing : public IUnknown
struct __declspec(uuid( "12345678-ABCD-1234-1122-334455667788")) IAnotherThing;
DEFINE_GUIDSTRUCT("12345678-ABCD-1234-1122-334455667788", IWholeThing);
DEFINE_PROPERTYKEY(IOneMoreThing,0x12345678,0xABCD,0x1234,0x11,0x22,0x33,0x44,0x55,0x66,0x77,0x88, 9);
DEFINE_API_PKEY(PKEY_ThingClsid, Thing , 0x12345678, 0xABCD, 0x1234, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x87654321);
DEFINE_PCI_ROOT_BUS_DEVPKEY(DEVPKEY_Thing, 0);